/*Rare Events Example*/

libname RDExamp "C:\users\sasdemo\RD_Examples";
%let test_env = rareevent;

/*------------------------------------------------*/
/*---- 1. Create the environment -----------------*/
/*------------------------------------------------*/

proc risk;
  env new=RDExamp.&test_env;
  env save;
run;

/*------------------------------------------------*/
/*---- 2. Register portfolio data ----------------*/
/*------------------------------------------------*/

data RDExamp.instdata;
  input insttype $12. instid $8. holding 8.;
  datalines;
TechCompany instid1 1000000
BankCompany instid2 1000000
;


proc risk;
  env open=RDExamp.&test_env;
  instdata Instruments file=RDExamp.instdata format=simple;
  env save;
run;


/*------------------------------------------------*/
/*---- 3. Register the market data ---------------*/
/*------------------------------------------------*/

data RDExamp.mktdata;
  TEC=15; 
  BNK=3;  
run;

data RDExamp.covdata;
	length _name_ _type_ $8;
	_type_ = "COV";
	input _name_ $ TEC BNK;  
	datalines;
TEC      .0004   .00006
BNK      .00006   .0001
;


/*Add Rare event: Need to define rare events as scenarios then create
rare event data set*/

proc risk;
	env open=RDExamp.&test_env;
	scenario BNKStress type=abs
          changes=( BNK -.50 );
    scenario TECStress type=rel
          changes=( TEC -.20);
	env save;
run;

data RDExamp.RareEventMap;
length eventname $32.;
input eventname $ uncond_prob cond_prob;
datalines;
BNKStress .20 .25
TECStress .05 .10
;


proc risk;
	env open=RDExamp.&test_env;
    declare riskfactors=(TEC num var, BNK num var);
	marketdata Market file=RDExamp.mktdata  type=current;
	marketdata Cov    file=RDExamp.covdata  type=covariance;
	marketdata RareEvents file = RDExamp.RareEventMap 
		type = rareevent(seed = 1234) interval = day;
	env save;
run;



/*------------------------------------------------*/
/*---- 4. Define valuation methods ---------------*/
/*------------------------------------------------*/

proc compile env=RDExamp.&test_env outlib=RDExamp.&test_env;
  method Tech_Price kind=price;
    _value_ = TEC;
  endmethod;
  method Bank_Price kind=price;
    _value_ = BNK;
  endmethod;
run;

proc risk;
  env open=RDExamp.&test_env;
  instrument TechCompany methods=(Price Tech_Price)
                         variables = (holding);
  instrument BankCompany methods=(Price Bank_Price)
                         variables = (holding);
  env save;
run; 


/*------------------------------------------------*/
/*---- 5. Create Project and Specify Analysis ----*/
/*------------------------------------------------*/
proc risk;
  env open=RDExamp.&test_env;
  sources InstDataSource (Instruments);
  read sources=InstDataSource out=Portfolio;
  crossclass RareEventCC (Insttype);
  simulation Covariance method=covariance
        ndraws=10000
        outvars=(pl)
        horizon = (1 2 3 4 5)
        data= (Cov RareEvents)
        ;
  project Project
        analysis=(Covariance)
        portfolio=Portfolio
        crossclass=RareEventCC
        data=( Market )
	  ;
      env save;
run;


/*------------------------------------------------*/
/*---- 6. Run Analysis ---------------------------*/
/*------------------------------------------------*/

proc risk;
  env open=RDExamp.&test_env;
  setoptions keeplib;
  runproject Project
      out = RareEvt
      options=(outall rareevents)
      ;
  env save;
run;



/*------------------------------------------------*/
/*---- 7. View Results ---------------------------*/
/*------------------------------------------------*/

/*You may need to assign a libref to the output directory*/
data rareevt.TECStress;
	do i = 1 to 10000;
	simulationreplication = i;
	do j = 1 to 5;
	simulationtime = j;
	output;
	end;
	end;
	drop i j;
run;

data rareevt.TECStress;
set rareevt.TECStress;
length eventname $20.;
if _n_ eq 1 then do;
declare hash h(dataset:'rareevt.rareevents');
	h.defineKey('simulationreplication' ,'simulationtime');
	h.defineData('eventname');
	h.defineDone();
end;
if h.find() eq 0 then output;
else do;
eventname = "Underlying Model";
output;
end;
run;

proc transpose data = rareevt.TECStress
out = rareevt.TECStressT
prefix = SimulationTime;
by simulationreplication;
id simulationtime;
var eventname;
run;

proc freq data = rareevt.tecstresst;
tables SimulationTime1/NoFreq NoCum;
tables SimulationTime1*SimulationTime2 SimulationTime2*SimulationTime3 SimulationTime3*SimulationTime4 SimulationTime4*SimulationTime5/NoFreq Nopercent NoCol ;
run;
